﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Collections;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_ProductionReceiptScript
{
    /// <summary>
    /// 
    /// </summary>
    public class WorkflowScript_TeoxaneProductionReceiptScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private static class Defs
        {
            /// <summary>
            /// The time to sleep in milliseconds
            /// </summary>
            public const int SleepTime = 2000;
        }

        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        ISboProviderService m_sboProviderService;

        // Output parameters
        // <none>

        // Sub-flows



        WorkflowInstanceBase m_enterQuantitySubFlow;

        protected bool m_subFlowBackRequested;
        protected Collection<string> m_subFlowListOfSerialNumbers;
        protected double m_subFlowQuantity;


        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_TeoxaneProductionReceiptScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_TeoxaneProductionReceiptScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_enterQuantitySubFlow = this.Create("EnterQuantityScript");

        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            int messageTime = 3;

            string deviceID = session.DeviceInstanceId;

            Produmex.Foundation.Messages.Message msg = null;

            string inputLocationCode = null;
            string productionLineCode = null;
            string outputLocationCode = null;
            string restLocationCode = null;
            int? prodOrderDocEntry = null;
            DataSet dsProductionOrders = null;
            DataSet dsLogisticCarriers = null;
            string logisticCarrierCode = null;
            string buttonKeyOtherTasksLogisticCarriers = null;
            string existingSSCC = null;
            int? existingLUID = null;

            string batchNumber = null;
            string batchNumber2 = null;
            DateTime? bestBeforeDate = null;
            string itemCode = null;
            PmxItemInfo itemInfo = null;
            double quantityToProduce = 0;
            string docNum = null;
            string logisticCarrierLocationCode = null;
            double? maximumQuantity = null;
            int numberOfReports = 0;
            bool askReprint = false;
            bool isDisassembly = false;
            double stillToQty = 0;

            Collection<PackagingTypeInfo> listOfPackagingTypeInfosForScreen = null;
            Collection<ItemTransactionalPackagingTypeInfo> currentListOfTransactionalPackagingTypeInfos;

            GeneratedBatchAndBestBeforeDateInfo genBatchBBDInfo = null;

            Collection<string> listOfSerialNumbers = null;


            object initialErrorKey = null;
            //ProdumexError PmxError = null;  // Never used

            bool confirmProducedQtyAfterProduction = m_sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null,
                delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                    return prov.ConfirmProducedQtyAfterProduction();
                });

            bool autoMoveAllItemsLinkedToBOM = false;
            bool allowToMoveFromRest = false;
            bool allowToStartPRD = false;
            bool splitQtyInLogUnitsOfItemDefQtyOnLogUnit = false;
            bool proceedWithCurrentProductionOrder = false;

            m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
              delegate(PmxDbConnection conn, object[] parameters)
              {

                  PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                  Dictionary<string, PmxParameterValue> plcParams = prdProv.GetParametersProductionController();

                  autoMoveAllItemsLinkedToBOM = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTO_MOVE_ITEMS_LINKED_TO_BOM, false);
                  allowToMoveFromRest = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_MOVE_TO_PROD_LINE_FROM_REST, false);
                  allowToStartPRD = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_ALLOW_START_PRODUCTION_RECEIPT, false);
                  splitQtyInLogUnitsOfItemDefQtyOnLogUnit = PmxParameterValueProvider.GetParameterValue<bool>( plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_SPLIT_QTY_IN_LOG_UNITS_OF_ITEM_DEF_QTY_ON_LOG_UNIT, false );
                  proceedWithCurrentProductionOrder = PmxParameterValueProvider.GetParameterValue<bool>( plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_PROCEED_WITH_CURRENT_PRD_ON_PRD_RECEIPT, false );

                  return null;
              });

            string buttonKeySerial = null;

            string query = null;
            int? scannedDocNum = null;

        Step_IdentifyProductionOrder:

            #region IDENTIFY PRODUCTION ORDER

            scannedDocNum = null;

            //user can change the batch, so ask it
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
             DefaultCultureInfo.Get(), BuildParamCollection(
                 "InitialErrorKey", initialErrorKey,
                 "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_ORDER",
                 "ForceDataEntry", false,
                 "ForceUppercase", false,
                 "AllowToGoBack", true,
                 "DefaultValue", null,
                 "Information", null,
                 "MessageAddendum", null,
                 "ButtonKey", "MSG_BTN_SELECT_PRODUCTION_ORDER"
                 ));
            msg = WaitForMessage();

            // reset error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringEntered"))
            {
                string scannedData = ExtractParameter<string>(msg.Parameters, "stringValue");

                int tempDocNum;

                if(!string.IsNullOrEmpty(scannedData) &&
                    int.TryParse(scannedData.Split(';')[0], out tempDocNum))
                {
                    scannedDocNum = tempDocNum;
                    goto Step_SelectProdOrder;
                }
                else
                {
                    initialErrorKey = "MSG_ERROR_INCORRECT_BARCODE";
                    goto Step_IdentifyProductionOrder;
                }
                
            }
            else if (msg.Name.EndsWith(".ButtonClicked"))
            {
                goto Step_SelectProdOrder;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                return;
            }

            #endregion

        Step_SelectProdOrder:

            #region SELECT PRODUCTION ORDER

            batchNumber = null;
            batchNumber2 = null;
            bestBeforeDate = null;
            itemCode = null;
            itemInfo = null;
            isDisassembly = false;
            stillToQty = 0;


            // Get list of production orders that are started
            query = BuildQuery.GetProductionOrders(this.PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, allowToStartPRD, null, scannedDocNum);
            dsProductionOrders = m_sboProviderService.RunView(false, null, null, query);

            initialErrorKey = null;
            prodOrderDocEntry = null;
            inputLocationCode = null;
            productionLineCode = null;
            docNum = null;
            logisticCarrierLocationCode = null;

            if (this.GetNumberOfRows(dsProductionOrders) == 1 &&
                scannedDocNum.HasValue)
            {
                //auto select
                prodOrderDocEntry = Convert.ToInt32(dsProductionOrders.Tables[0].Rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name]);
            }
            else
            {
                // Show screen with list
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderByDocEntryScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_ORDER",
                        "ProductionOrderDS", dsProductionOrders));
                msg = WaitForMessage();

                

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    goto Step_IdentifyProductionOrder;
                }
                if (msg.Name.EndsWith(".RefreshRequested"))
                {
                    goto Step_SelectProdOrder;
                }
                if (!msg.Name.EndsWith(".ProductionOrderSelected"))
                {
                    initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                    goto Step_SelectProdOrder;
                }

                prodOrderDocEntry = ExtractParameter<int>(msg.Parameters, "productionOrderDocEntry");

            }

            DataRow[] prodOrderRows = dsProductionOrders.Tables[ 0 ].Select(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name + "=" + prodOrderDocEntry.ToString());
            if ( prodOrderRows.Length < 1 )
            {
                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectProdOrder;
            }

            inputLocationCode = Convert.ToString(prodOrderRows[0][PmxOseProductionLineTable.Columns.InputLocationCode.NAME]);
            productionLineCode = Convert.ToString(prodOrderRows[0][ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name]);
            outputLocationCode = Convert.ToString(prodOrderRows[0][PmxOseProductionLineTable.Columns.OutputLocationCode.NAME]);
            restLocationCode = Convert.ToString(prodOrderRows[0][PmxOseProductionLineTable.Columns.RestLocationCode.NAME]);

            itemCode = Convert.ToString(prodOrderRows[0][ProductionOrderDataSet.Columns.ITEMCODE.Name]);
            docNum = Convert.ToString(prodOrderRows[0][ProductionOrderDataSet.Columns.DOC_NUM.Name]);

            //get iteminfo
            itemInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode);

            isDisassembly = Convert.ToString(prodOrderRows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) == SboProductionOrderTypeValues.Disassembly.VALUE;

            stillToQty = Convert.ToDouble(prodOrderRows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);


            if (allowToStartPRD && 
                Convert.ToString(prodOrderRows[0][ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name]) != PmxProductionStatusColumnValues.Started.VALUE)
            {
            Step_StartPRD:

                try
                {
                    //start prd
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                     delegate(PmxDbConnection conn, object[] parameters)
                     {

                         PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                         prdProv.StartProductionOrder(prodOrderDocEntry.Value);



                         //set auto line up locations
                         PmxLinedUpLocationProvider locProv = new PmxLinedUpLocationProvider((SboConnection)conn);

                         // * Auto line-up *
                         string qry = BuildQuery.GetQueryGetProdOrderLinedUpLocations(prodOrderDocEntry.Value, true, conn.DbCultureInfo);
                         using (PmxProdOrderLineUpLocations rsLinedUpItems = SboRecordsetHelper.RunQuery<PmxProdOrderLineUpLocations>(s_log, qry, conn))
                         {
                             while (!rsLinedUpItems.EoF)
                             {
                                 string itemCodeLineUp = rsLinedUpItems.ItemCode;

                                 qry = BuildQuery.GetQueryGetCanBeLinedUpLocations(productionLineCode, itemCodeLineUp, prodOrderDocEntry.Value, conn.DbCultureInfo);
                                 using (PmxProdOrderLineUpLocations rsLinedUpSources = SboRecordsetHelper.RunQuery<PmxProdOrderLineUpLocations>(s_log, qry, conn))
                                 {
                                     if (!rsLinedUpSources.EoF)
                                     {
                                         Collection<PmxLinedUpLocation> lineUps = locProv.GetLinedUpLocationsForProductionLine(productionLineCode, rsLinedUpItems.ProdOrderLineNum.Value, prodOrderDocEntry.Value);

                                         if (lineUps == null || lineUps.Count < 1)
                                         {
                                             using (PmxLinedUpLocation newLineUp = locProv.GetNewBO())
                                             {
                                                 newLineUp.ProductionLineCode = productionLineCode;
                                                 newLineUp.LinedUpLocationCode = rsLinedUpSources.TankCode;
                                                 newLineUp.ProductionOrderLineNum = rsLinedUpItems.ProdOrderLineNum.Value;
                                                 newLineUp.ProductionOrderDocEntry = prodOrderDocEntry.Value;

                                                 locProv.AddBO(newLineUp);
                                             }
                                         }
                                         else
                                         {
                                             lineUps[0].LinedUpLocationCode = rsLinedUpSources.TankCode;

                                             locProv.UpdateBO(lineUps[0], false, false);
                                         }
                                     }
                                     else
                                     {
                                         throw new ProdumexException(Produmex.Sbo.Logex.Data.ProdumexErrors.AutoLineUpNoLocationFound(s_log, itemCodeLineUp));

                                     }
                                 }

                                 rsLinedUpItems.MoveNext();
                             }
                         }


                         return null;
                     });

                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ex.Message,
                                "NoTranslationOfMessageKey", true,
                                "IsError", true, 
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_StartPRD;
                    }

                    goto Step_IdentifyProductionOrder;
                }
            }


            //move to prd line if needed
            if (autoMoveAllItemsLinkedToBOM &&
                prodOrderDocEntry.HasValue)
            {

                #region MOVE ALL ITEMS LINKED TO BOM

            Step_MoveStockToPRD:

                try
                {

                    //get all itemcodes for production order
                    Collection<string> listOfItemCodesToMove = new Collection<string>();


                    if (!isDisassembly)
                    {

                        //get list of itemcodes
                        query = BuildQuery.GetProductsFromStockLinkedToProductionOrder(prodOrderDocEntry.Value, inputLocationCode, allowToMoveFromRest ? restLocationCode : null);
                        DataSet dsItems = m_sboProviderService.RunView(true, null, null, query);

                        if (GetNumberOfRows(dsItems) > 0)
                        {
                            foreach (DataRow row in dsItems.Tables[0].Rows)
                            {
                                string itemCodeToMove = Convert.ToString(row[ProductDataSet.Columns.PRODUCT_CODE.Name]);

                                if (!listOfItemCodesToMove.Contains(itemCodeToMove))
                                {
                                    listOfItemCodesToMove.Add(itemCodeToMove);
                                }

                            }
                        }
                    }
                    else
                    {
                        //add main item
                        listOfItemCodesToMove.Add(itemCode);
                    }


                    //move all data linked to production order
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                         delegate(PmxDbConnection conn, object[] parameters)
                         {
                             PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
                             prov.MoveItemsFromInputToProductionLine(productionLineCode, listOfItemCodesToMove, allowToMoveFromRest, prodOrderDocEntry);
                             return null;
                         });

                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ex.Message,
                                "NoTranslationOfMessageKey", true,
                                "IsError", true, 
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_MoveStockToPRD;
                    }


                }


                #endregion



            }


            //get batch info from controller!

            genBatchBBDInfo = m_sboProviderService.InvokeMethodWithDbConnection<GeneratedBatchAndBestBeforeDateInfo>(false, false, null, null, new DelegateWithPmxDbConnection<GeneratedBatchAndBestBeforeDateInfo>(ExecuteGetBatchAndBestBeforeDate), prodOrderDocEntry);



            /*
            batchNumber = Convert.ToString(prodOrderRows[0][PmxProductionOrderTable.Columns.BatchNumber.NAME]);
            batchNumber2 = Convert.ToString(prodOrderRows[0][PmxProductionOrderTable.Columns.BatchNumber2.NAME]);

            if (prodOrderRows[0][PmxProductionOrderTable.Columns.BestBeforeDate.NAME] is DateTime)
            {
                bestBeforeDate = Convert.ToDateTime(prodOrderRows[0][PmxProductionOrderTable.Columns.BestBeforeDate.NAME]);
            }
             */



            //get location for logistic carriers
            logisticCarrierLocationCode = m_sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null, new DelegateWithPmxDbConnection<string>(ExecuteGetStorageLocationLogisticCarriers), inputLocationCode);


            //Already check the maximum quantity for the production order
            maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null, new DelegateWithPmxDbConnection<double?>(ExecuteGetMaximumQuantity), prodOrderDocEntry, !string.IsNullOrEmpty(logisticCarrierLocationCode), logisticCarrierLocationCode, itemInfo);

            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value == 0)
            {
                //no quantity allowed
                initialErrorKey = "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE";
                goto Step_IdentifyProductionOrder;
            }

            goto Step_EnterBatch;


            #endregion

        Step_EnterBatch:

            #region ENTER BATCH

            if (itemInfo.HasBatchnumber)
            {
                batchNumber = genBatchBBDInfo.BatchNumber;

                if (genBatchBBDInfo.IsUserAllowedToChangeBatchNumber)
                {
                    //user can change the batch, so ask it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER_FOR_PRODUCTION",
                         "ForceDataEntry", true,
                         "ForceUppercase", true,
                         "AllowToGoBack", true,
                         "DefaultValue", batchNumber,
                         "Information", "MSG_DEFAULT_BATCH_NUMBER",
                         "MessageAddendum", genBatchBBDInfo.BatchNumber
                         ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".StringEntered"))
                    {
                        batchNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goto Step_IdentifyProductionOrder;
                    }
                }

            }

            goto Step_EnterBatch2;

            #endregion

        Step_EnterBatch2:

            #region ENTER BATCH2

            if (itemInfo.HasSecondBatchNumber)
            {
                batchNumber2 = genBatchBBDInfo.BatchNumber2;

                if (genBatchBBDInfo.IsUserAllowedToChangeBatchNumber2)
                {
                    //user can change the batch, so ask it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER2_FOR_PRODUCTION",
                         "ForceDataEntry", true,
                         "ForceUppercase", true,
                         "AllowToGoBack", true,
                         "DefaultValue", batchNumber2,
                         "Information", "MSG_DEFAULT_BATCH_NUMBER",
                         "MessageAddendum", genBatchBBDInfo.BatchNumber2
                         ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".StringEntered"))
                    {
                        batchNumber2 = ExtractParameter<string>(msg.Parameters, "stringValue");
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        if (genBatchBBDInfo.IsUserAllowedToChangeBatchNumber)
                        {
                            goto Step_EnterBatch;
                        }

                        goto Step_IdentifyProductionOrder;
                    }
                }

            }

            goto Step_EnterBBD;

            #endregion

        Step_EnterBBD:

            #region ENTER BBD

            if (itemInfo.HasBestBeforeDate)
            {
                bestBeforeDate = genBatchBBDInfo.BestBeforeDate;

                if (genBatchBBDInfo.IsUserAllowedToChangeBestBeforeDate)
                {
                    //user can change the batch, so ask it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterDateScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE_FOR_PRODUCTION",
                         "ForceDataEntry", true,
                         "ForceUppercase", true,
                         "AllowToGoBack", true,
                         "DefaultDate", bestBeforeDate,
                         "Information", "MSG_DEFAULT_BEST_BEFORE_DATE"
                         ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".DateEntered"))
                    {
                        bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "date");
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        if (genBatchBBDInfo.IsUserAllowedToChangeBatchNumber2)
                        {
                            goto Step_EnterBatch2;
                        }
                        if (genBatchBBDInfo.IsUserAllowedToChangeBatchNumber)
                        {
                            goto Step_EnterBatch;
                        }

                        goto Step_IdentifyProductionOrder;
                    }
                }

            }

            goto Step_SelectLogisticCarrier;


            #endregion

        Step_SelectLogisticCarrier:

            #region SELECT LOGISTIC CARRIERS


            logisticCarrierCode = null;
            existingSSCC = null;
            existingLUID = null;
            bool existingLUIDIsInStock = false;
            double advanceNoticeQty = 0;


            query = BuildQuery.GetLogisticCarriers(true, true, m_sboProviderService.GetDbTool());
            dsLogisticCarriers = m_sboProviderService.RunView(false, null, null, query);

            if (!string.IsNullOrEmpty(itemInfo.DefaultLogisticCarrierProduction) &&
               !isDisassembly)
            {
                logisticCarrierCode = itemInfo.DefaultLogisticCarrierProduction;
                goto Step_OtherTasksLogisticCarrier;
            }

            if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
            {
                //We found logistic carriers, so we ask to select

                // Show screen with list
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
                        "ProductDS", dsLogisticCarriers,
                        "ButtonKey", "MSG_BUTTON_OTHER_TASK"));
                msg = WaitForMessage();

                initialErrorKey = null;

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    goto Step_IdentifyProductionOrder;
                }
                if (msg.Name.EndsWith(".RefreshRequested"))
                {
                    goto Step_SelectLogisticCarrier;
                }
                if (msg.Name.EndsWith(".ButtonClicked"))
                {
                    goto Step_OtherTasksLogisticCarrier;
                }
                if (!msg.Name.EndsWith(".ProductSelected"))
                {
                    initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                    goto Step_SelectLogisticCarrier;
                }

                logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

                goto Step_EnterPackagingQuantity;

                
            }
            else
            {
                goto Step_OtherTasksLogisticCarrier;
            }

            #endregion

        Step_OtherTasksLogisticCarrier:

            #region OTHER TASKS LOGISTIC CARRIERS

            buttonKeyOtherTasksLogisticCarriers = null;

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_TASK",
                    "AllowedButtons", new string[] { string.IsNullOrEmpty(logisticCarrierCode) ? "MSG_BUTTON_NO_LOGISTIC_CARRIER" : "MSG_BUTTON_PROCEED", "MSG_BUTTON_PRODUCE_ON_EXISTING_SSCC" }));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
                {
                    goto Step_SelectLogisticCarrier;
                }
                else
                {
                    goto Step_IdentifyProductionOrder;
                }
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeyOtherTasksLogisticCarriers = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_NO_LOGISTIC_CARRIER" ||
                    buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_PROCEED")
                {
                    goto Step_EnterPackagingQuantity;
                }
                else
                {
                    goto Step_SelectExistingSSCC;
                }

            }

            #endregion

        Step_SelectExistingSSCC:

            #region SELECT EXISTING SSCC

            existingSSCC = null;
            existingLUID = null;
            existingLUIDIsInStock = false;
            advanceNoticeQty = 0;

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IIdentifySSCCScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_IDENTIFY_SSCC"));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_OtherTasksLogisticCarrier;
            }
            if ( msg.Name.EndsWith( ".SSCCEntered" ) )
            {
                existingSSCC = ExtractParameter<string>( msg.Parameters, "sscc" );

                //query = BuildQuery.GetProductOnSSCC(existingSSCC, null);
                query = BuildQuery.GetInfoForInStockOrAdvanceNoticeSSCC( existingSSCC );
                DataSet ds = m_sboProviderService.RunView( false, null, null, query );

                if ( this.GetNumberOfRows( ds ) < 1 )
                {
                    initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                    goto Step_SelectExistingSSCC;
                }

                existingLUID = SystemInt32Convertor.ObjectToIntOrNull( ds.Tables[ 0 ].Rows[ 0 ][ PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ] );
                if ( existingLUID.HasValue )
                {
                    existingLUIDIsInStock = true;
                }
                else
                {
                    existingLUID = SystemInt32Convertor.ObjectToIntOrNull( ds.Tables[ 0 ].Rows[ 0 ][ PmxAdvanceShippingNoticeTable.NAME + "_" + PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME ] );
                }
                advanceNoticeQty = SystemDoubleConvertor.ObjectToDouble( ds.Tables[ 0 ].Rows[ 0 ][ PmxAdvanceShippingNoticeTable.NAME + "_" + PmxAdvanceShippingNoticeTable.Columns.Quantity.NAME ], 0, 0 );

                if ( !existingLUIDIsInStock )
                {
                    goto Step_WithAdvanceNoticeSelectLogisticCarrier;
                }

                //clear logistic carrier if it was set by default logistic carrier
                logisticCarrierCode = null;
                goto Step_EnterPackagingQuantity;
            }


            #endregion
            
        Step_WithAdvanceNoticeSelectLogisticCarrier:

            #region WithAdvanceNoticeSelectLogisticCarrier

            logisticCarrierCode = null;

            //query = BuildQuery.GetLogisticCarriers( true, true );
            //dsLogisticCarriers = m_sboProviderService.RunView( false, null, null, query );  // Already filled in

            if ( this.GetNumberOfRows( dsLogisticCarriers ) > 0 )
            {
                //We found logistic carriers, so we ask to select

                // Show screen with list
                session.ShowScreen( typeof( Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen ),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
                        "ProductDS", dsLogisticCarriers ) );
                msg = WaitForMessage();

                initialErrorKey = null;

                if ( msg.Name.EndsWith( ".BackRequested" ) )
                {
                    goto Step_IdentifyProductionOrder;
                }
                if ( msg.Name.EndsWith( ".RefreshRequested" ) )
                {
                    goto Step_WithAdvanceNoticeSelectLogisticCarrier;
                }
                if ( !msg.Name.EndsWith( ".ProductSelected" ) )
                {
                    initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                    goto Step_SelectLogisticCarrier;
                }

                logisticCarrierCode = ExtractParameter<string>( msg.Parameters, "itemCode" );
            }

            goto Step_EnterPackagingQuantity;

            #endregion

        Step_EnterPackagingQuantity:

            #region ENTER PACKAGING QUANTITY


            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value == 0)
            {
                //no quantity allowed
                initialErrorKey = "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE_QUANTITY";
                goto Step_IdentifyProductionOrder;
            }

            if (isDisassembly)
            {
                currentListOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();
                goto Step_EnterQuantity;
            }

            if ((itemInfo.HasBatchnumber || itemInfo.HasSecondBatchNumber || itemInfo.HasBestBeforeDate))
            {

                //batch not found, so we need to enter the quantity for the packaging types
                currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfo);
                bool foundAtLeastOnePackagingTypeToShowOnScreen = false;
                bool isFirst = true;
                listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();
                foreach (PmxItemPackagingTypeInfo packType in itemInfo.ListOfPackagingTypes)
                {

                    PackagingTypeInfo info = new PackagingTypeInfo();

                    if (isFirst)
                    {
                        info.NumberOfDecimals = itemInfo.UomDecimals;
                    }

                    info.Quantity = packType.Quantity;
                    info.PackagingTypeCode = packType.PackagingTypeCode;
                    info.PackagingTypeName = packType.PackagingTypeName;

                    info.ShowOnScreen = packType.AskDuringReception;

                    listOfPackagingTypeInfosForScreen.Add(info);

                    if (!foundAtLeastOnePackagingTypeToShowOnScreen &&
                        info.ShowOnScreen)
                    {
                        foundAtLeastOnePackagingTypeToShowOnScreen = true;
                    }

                    isFirst = false;
                }

                if (foundAtLeastOnePackagingTypeToShowOnScreen)
                {
                    initialErrorKey = "MSG_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES";
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES",
                        "ItemDescription", itemInfo.Description,
                        "ItemCode", itemInfo.ItemCode,
                        "ItemBarcode", itemInfo.CodeBars, "UOM", itemInfo.InventoryUom,
                        "ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
                        "IsForEnteringDefaultPackagingQuantity", true,
                        "BatchNumber", batchNumber,
                        "BestBeforeDate", bestBeforeDate,
                        "SSCC", existingSSCC,
                        "DocNum", docNum
                        ));

                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".QuantitiesEntered"))
                    {
                        PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

                        //build the list of packaging quantities
                        foreach (ItemTransactionalPackagingTypeInfo info in currentListOfTransactionalPackagingTypeInfos)
                        {
                            //try to get new qty from user
                            if (screenListOfPackagingTypeInfos.PackagingTypeInfoList.ContainsKey(info.PackagingTypeCode))
                            {
                                info.Quantity = screenListOfPackagingTypeInfos.PackagingTypeInfoList[info.PackagingTypeCode].Quantity;
                            }
                        }

                        goto Step_EnterQuantity;
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goto Step_SelectLogisticCarrier;
                    }

                }

                goto Step_EnterQuantity;
            }
            else
            {
                //no batch info
                //get the packaging types qty from item
                currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfo);

              goto Step_EnterQuantity;
            }

            #endregion

        Step_EnterQuantity:

            #region ENTER QUANTITY

            buttonKeySerial = null;

        Step_EnterSerialNumbersReEntryOnProceed:


            quantityToProduce = 0;

            listOfSerialNumbers = null;
            m_subFlowListOfSerialNumbers = null;

            bool getNormalQuantity = true;

            if ((itemInfo.HasSerialNumber || itemInfo.HasSAPSerialNumber) &&
                !isDisassembly)
            {
                #region SERIAL NUMBER

                getNormalQuantity = false;

                if (string.IsNullOrEmpty(buttonKeySerial))
                {

                    //show screen to ask user what to do.
                    List<string> buttonKeys = new List<string>(new Collection<string>() { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" });

                    if (!(itemInfo.TrackLocationSerialNumber ||
                        (itemInfo.HasSAPSerialNumber && itemInfo.ManageMethod == SboItemManageMethod.OnEveryTransaction)))
                    {
                        buttonKeys.Add("MSG_BUTTON_NO_SERIAL_NUMBER");
                    }


                    ToolBarButtons toolBarButtons = ToolBarButtons.Back;

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
                            "AllowedButtons", buttonKeys.ToArray(),
                            "EnabledToolBarButtons", toolBarButtons
                            ));
                    msg = WaitForMessage();
                }

                initialErrorKey = null;
                if (msg.Name.EndsWith(".ButtonClicked") || 
                    !string.IsNullOrEmpty(buttonKeySerial))
                {
                    if (string.IsNullOrEmpty(buttonKeySerial))
                    {
                        buttonKeySerial = ExtractParameter<string>(msg.Parameters, "buttonKey");
                    }

                    if (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER" ||
                        buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
                    {
                        #region ENTER SERIAL NUMBERS

                        //get ITRI
                        int? itriKey = null;
                        if (itemInfo.HasBestBeforeDate ||
                            itemInfo.HasSecondBatchNumber ||
                            itemInfo.HasBatchnumber)
                        {
                            itriKey = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecuteGetCreateItemTransactionalInfoKey), itemInfo.ItemCode, batchNumber, batchNumber2, bestBeforeDate);
                        }


                        bool isSerialNumberRange = (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

                        m_enterQuantitySubFlow.BindParameters(
                            this.DefaultCultureInfo,
                            this.PmxOseCompany,
                            new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
                            new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
                            new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                            new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
                            new ReadOnlyBinder<int?>("ItriKey", itriKey),
                            new ReadOnlyBinder<string>("LocationCode", productionLineCode),
                            new ReadOnlyBinder<string>("SSCC", null),

                            new ReadOnlyBinder<string>("DocNum", docNum.ToString()),
                            new ReadOnlyBinder<bool>("UsePickingInfo", false),

                            new ReadOnlyBinder<bool>("AllowZero", false),
                            new ReadOnlyBinder<double>("InitialQuantity", 1),
                            new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"),
                            new ReadOnlyBinder<double>("StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0),
                            new ReadOnlyBinder<double>("MaximumQuantity", maximumQuantity.Value),
                            new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
                            new ReadOnlyBinder<string>("UOM", itemInfo.InventoryUom),
                            new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfo != null) ? itemInfo.UomDecimals : 0),

                            new ReadOnlyBinder<double>("InnerQuantity", 1),
                            new ReadOnlyBinder<string>("InnerUOM", ""),
                            new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
                            new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.ProductionOrder).ToString()),
                            new ReadOnlyBinder<int?>("DocEntry", Convertor.ConvertToInt32Nullable(docNum)),
                            new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
                            new ReadOnlyBinder<string>("LocationCode", productionLineCode),

                            new ReadOnlyBinder<int?>("LUID", null),


                            new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
                            new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
                            new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

                            );

                        m_subFlowBackRequested = false;
                        this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

                        if (m_subFlowBackRequested)
                        {
                            goto Step_SelectLogisticCarrier;
                        }

                        listOfSerialNumbers = m_subFlowListOfSerialNumbers;

                        quantityToProduce = m_subFlowQuantity;

                        goto Step_ConfirmQuantity;

                        #endregion

                    }

                    getNormalQuantity = true;
                }

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    goto Step_SelectLogisticCarrier;
                }



                #endregion
            }

            if (getNormalQuantity)
            {

                if (itemInfo.ListOfPackagingTypes != null &&
                    currentListOfTransactionalPackagingTypeInfos.Count > 0)
                {

                    //build list of packaging types
                    listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();

                    //add first one for the uom

                    PackagingTypeInfo info = new PackagingTypeInfo();

                    info.Quantity = 1;
                    info.PackagingTypeCode = null;
                    info.PackagingTypeName = itemInfo.InventoryUom == null ? itemInfo.ItemCode : itemInfo.InventoryUom;
                    info.QuantityPerPack = 1;
					info.Barcode = new Collection<string>() { itemInfo.CodeBars };
                    info.NumberOfDecimals = itemInfo.UomDecimals;
                    info.InitialQuantity = 1;
                    listOfPackagingTypeInfosForScreen.Add(info);


                    foreach (ItemTransactionalPackagingTypeInfo packType in currentListOfTransactionalPackagingTypeInfos)
                    {
                        info = new PackagingTypeInfo();

                        info.Quantity = packType.Quantity;
                        info.PackagingTypeCode = packType.PackagingTypeCode;
                        info.PackagingTypeName = GetPackTypeName(packType.PackagingTypeCode, itemInfo.ListOfPackagingTypes);
						info.Barcode = GetPackBarcode(packType.PackagingTypeCode, itemInfo.ListOfPackagingTypes);


                        info.InitialQuantity = 0;

                        info.QuantityPerPack = packType.Quantity;

                        listOfPackagingTypeInfosForScreen.Add(info);
                    }


                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "TitleKey", "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
                                        "ItemDescription", itemInfo.Description,
                                        "ItemCode", itemInfo.ItemCode,
                                        "ItemBarcode", itemInfo.CodeBars,
                                        "UOM", itemInfo.InventoryUom,
                                        "ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
                                        "IsForEnteringDefaultPackagingQuantity", false,
                                        "BatchNumber", batchNumber,
                                        "BestBeforeDate", bestBeforeDate,
                                        "SSCC", existingSSCC,
                                        "DocNum", docNum,
                                        "StillToQuantity", stillToQty,
                        //"StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0,
                        //"MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY",
                                        "AllowZero", false,
                                        "MaximumQuantity", maximumQuantity.Value
                                        ));

                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".QuantitiesEntered"))
                    {
                        PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

                        //build the list of packaging quantities
                        foreach (PackagingTypeInfo infoForQty in screenListOfPackagingTypeInfos.PackagingTypeInfoList.Values)
                        {
                            quantityToProduce += (infoForQty.Quantity * infoForQty.QuantityPerPack);
                        }


                        goto Step_Produce;
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goto Step_SelectLogisticCarrier;
                    }
                }
                else
                {

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", isDisassembly ? "MSG_TITLE_ENTER_QUANTITY_TO_DISASSEMBLE" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
                        "InitialQuantity", Math.Max(1.0, itemInfo.DefaultQuantityOnLogisticUnit),
                        "NumberOfDigitsAfterDecimal", itemInfo.UomDecimals,
                        "MaximumQuantity", maximumQuantity.Value,
                        "UOM", itemInfo.InventoryUom,
                        "DocNum", docNum,
                        "ItemDescription", itemInfo.Description,
                        "ItemCode", itemCode,
                        "ItemBarcode", itemInfo.CodeBars,
                        "BatchNumber", batchNumber,
                        "BestBeforeDate", bestBeforeDate,
                        "StillToQuantity", stillToQty
                        //"StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0,
                        //"MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"
                        ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".QuantityEntered"))
                    {
                        quantityToProduce = ExtractParameter<double>(msg.Parameters, "quantity");
                        goto Step_ConfirmQuantity;
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goto Step_SelectLogisticCarrier;

                    }

                }
            }



            #endregion

        Step_ConfirmQuantity:

            #region CONFIRM QUANTITY

            if (confirmProducedQtyAfterProduction && !isDisassembly)
            {
                //check if qty deviates from default qty
                if (
                    itemInfo.DefaultProductionQuantity > 0 &&
                    (quantityToProduce > itemInfo.DefaultProductionQuantity + (itemInfo.DefaultProductionQuantity * itemInfo.AllowedProductionDeviationPercentage / 100) ||
                    quantityToProduce < itemInfo.DefaultProductionQuantity - (itemInfo.DefaultProductionQuantity * itemInfo.AllowedProductionDeviationPercentage / 100))
                    )
                {

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_CONFIRM_PROD_QTY_AFTER_PROD",
                            "MessageKey", "MSG_QUESTION_CONFIRM_PROD_QTY_AFTER_PROD",
                            "Parameters", new object[] { quantityToProduce, itemInfo.Description }));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_Produce;
                    }

                    goto Step_EnterQuantity;
                }
            }

            goto Step_Produce;


            #endregion

        /*
        Step_EnterQuantity:

            #region ENTER QUANTITY

            

            quantityToProduce = 0;
            maximumQuantity = sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null, new DelegateWithPmxDbConnection<double?>(ExecuteGetMaximumQuantity), prodOrderDocEntry, !string.IsNullOrEmpty(logisticCarrierLocationCode), logisticCarrierLocationCode, itemInfo);

            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value == 0)
            {
                //no quantity allowed
                initialErrorKey = "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE_QUANTITY";
                goto Step_SelectProdOrder;
            }

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
                    "InitialQuantity", 1,
                    "NumberOfDigitsAfterDecimal", itemInfo.UomDecimals,
                    "MaximumQuantity", maximumQuantity,
                    "UOM", itemInfo.InventoryUom,
                    "DocNum", docNum,
                    "ItemDescription", itemInfo.Description,
                    "ItemCode", itemCode,
                    "ItemBarcode", itemInfo.CodeBars,
                    "BatchNumber", batchNumber,
                    "BestBeforeDate", bestBeforeDate
                    ));

            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectLogisticCarrier;
            }

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                quantityToProduce = ExtractParameter<double>(msg.Parameters, "quantity");
                goto Step_Produce;
            }


            #endregion
             
             */

        Step_Produce:

            #region PRODUCE

            List<InventoryItemInfo> producedItems = new List<InventoryItemInfo>();
            try
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                    delegate(PmxDbConnection conn, object[] parameters)
                    {
                        if (genBatchBBDInfo != null)
                        {
                            genBatchBBDInfo.BatchNumber = batchNumber;
                            genBatchBBDInfo.BatchNumber2 = batchNumber2;
                            genBatchBBDInfo.BestBeforeDate = bestBeforeDate;
                        }

                        double splitQty = quantityToProduce;
                        int splitInNrOfLogUnits = 1;
                        if (splitQtyInLogUnitsOfItemDefQtyOnLogUnit && itemInfo.DefaultQuantityOnLogisticUnit > 0
                            && (!existingLUID.HasValue || !existingLUIDIsInStock) && advanceNoticeQty == 0)
                        {
							splitInNrOfLogUnits = (int)Produmex.Foundation.Utilities.FormattingHelper.CeilingForDecimals(splitQty / itemInfo.DefaultQuantityOnLogisticUnit, 0);
						}

                        Collection<string> fullListOfSerialNumbers = listOfSerialNumbers;

                        for (int splitLogUnitIdx = 0; splitLogUnitIdx < splitInNrOfLogUnits; splitLogUnitIdx++)
                        {
                            double logUnitQty = splitQty;

                            if (splitInNrOfLogUnits > 1)
                            {
                                logUnitQty = Math.Min(splitQty, itemInfo.DefaultQuantityOnLogisticUnit);
                                splitQty = Math.Round(splitQty - logUnitQty, itemInfo.UomDecimals);
                            }

                            Collection<string> tempListOfSerialNumbers = null;

                            if (fullListOfSerialNumbers != null && fullListOfSerialNumbers.Count > 0)
                            {
                                tempListOfSerialNumbers = new Collection<string>();

                                for (int iLoop = 0; iLoop < logUnitQty; iLoop++)
                                {
                                    tempListOfSerialNumbers.Add(fullListOfSerialNumbers[iLoop]);
                                }

                            }

                            //produce
                            producedItems.Add(ExecuteProduceAndLock(conn, prodOrderDocEntry.Value, logUnitQty, logisticCarrierLocationCode, logisticCarrierCode, existingLUID, itemCode, outputLocationCode, currentListOfTransactionalPackagingTypeInfos, genBatchBBDInfo, tempListOfSerialNumbers));

                            if (tempListOfSerialNumbers != null)
                            {
                                //remove serial numbers from full list, so next run will get new serial numbers
                                foreach (string serNum in tempListOfSerialNumbers)
                                {
                                    fullListOfSerialNumbers.Remove(serNum);
                                }
                            }
                        }


                        return null;

                    });


            }
            catch (Exception ex)
            {
                if (!(ex is ProdumexException))
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ex.Message,
                            "NoTranslationOfMessageKey", true,
                            "IsError", true, 
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();

                }
                //error occured, so go back
                goto Step_IdentifyProductionOrder;
            }

            goto Step_PrintLabel;

            #endregion

        Step_PrintLabel:

            #region PRINT LABEL
            numberOfReports = 0;

            if ( !existingLUIDIsInStock && advanceNoticeQty > 0 )
            {
                // For LUID not in stock and advance notice exists, do not print label (is already printed)
                goto Step_AfterProduced;
            }

            try
            {
                foreach ( InventoryItemInfo item in producedItems )
                {
                    if ( item.LogisticUnitIdentKey != null )
                    {
                        //trigger event
                        numberOfReports += m_sboProviderService.InvokeMethodWithDbConnection<int>( false, false, null, null,
                        delegate( PmxDbConnection conn, object[] parameters2 )
                        {
                            PmxPrintReportProvider printProv = new PmxPrintReportProvider( conn );
							return printProv.PrintReportsForEvent(PmxPrintReportEventType.ProductionLogisticUnitProduced,
								productionLineCode, deviceID, item.LogisticUnitIdentKey.Value, out askReprint, null);
                        } );
                    }
                }
            }
            catch (Produmex.Foundation.ProdumexException ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", ex.Error,
                        "ShowButton", true));

                msg = WaitForMessage();
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                        "ShowButton", true));

                msg = WaitForMessage();
            }


            if (numberOfReports > 0)
            {
                goto Step_DocumentsPrinted;
            }

            //Produced items
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "MessageKey", isDisassembly ? "MSG_ITEM_IS_DISASSEMBLED" : "MSG_ITEM_IS_PRODUCED",
                   "ShowButton", true,
                   "MaxTimeForMessage", messageTime));

            this.WaitForMessage();

            goto Step_AfterProduced;

            #endregion

        Step_DocumentsPrinted:

            #region DOCUMENTS PRINTED

            if (numberOfReports > 0 &&
                askReprint)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                        "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintLabel;
                }

            }

            goto Step_AfterProduced;

            #endregion

        Step_AfterProduced:

            #region AFTER PRODUCED


            if (proceedWithCurrentProductionOrder)
            {
                // Get list of production orders that are started
                query = BuildQuery.GetProductionOrders(this.PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, allowToStartPRD, prodOrderDocEntry, null);
                dsProductionOrders = m_sboProviderService.RunView(false, null, null, query);

                stillToQty = 0;
                if (this.GetNumberOfRows(dsProductionOrders) > 0)
                {
                    stillToQty = Convert.ToDouble(dsProductionOrders.Tables[0].Rows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);
                }

                //Already check the maximum quantity for the production order
                maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null, new DelegateWithPmxDbConnection<double?>(ExecuteGetMaximumQuantity), prodOrderDocEntry, !string.IsNullOrEmpty(logisticCarrierLocationCode), logisticCarrierLocationCode, itemInfo);

                if (stillToQty > 0 && 
                    maximumQuantity.HasValue &&
                    maximumQuantity.Value != 0)
                {
                    // Inform the user he can input next logistic unit
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ENTER_NEXT_LOGISTIC_UNIT",
                            "ShowButton", true,
                            "MaxTimeForMessage", messageTime));
                    WaitForMessage();

                    if ((itemInfo.HasSerialNumber || itemInfo.HasSAPSerialNumber) &&
                        !isDisassembly)
                    {
                        goto Step_EnterSerialNumbersReEntryOnProceed;
                    }
                    else
                    {
                        goto Step_EnterQuantity;
                    }

                    //goto Step_SelectLogisticCarrier;
                }
            }

            goto Step_IdentifyProductionOrder;


            #endregion

        }

        #endregion


        #region HELPERS

        private static int? ExecuteGetCreateItemTransactionalInfoKey(PmxDbConnection conn, object[] parameters)
        {
            PmxItemTransactionalInfoProvider prov = new PmxItemTransactionalInfoProvider(conn);
            string itemCode = (string)parameters[0];
            string batchNumber = (string)parameters[1];
            string batchNumber2 = (string)parameters[2];
            DateTime? bbd = (DateTime?)parameters[3];

            return prov.GetCreateItemTransactionalInfoKey(itemCode, batchNumber, batchNumber2, bbd);
        }

        private static string GetPackTypeName(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
        {
            if (listOfPackagingTypes != null)
            {
                foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
                {
                    if (packType.PackagingTypeCode == packagingTypeCode)
                    {
                        return packType.PackagingTypeName;
                    }
                }
            }

            return null;
        }

        private static Collection<string> GetPackBarcode(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
        {
            if (listOfPackagingTypes != null)
            {
                foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
                {
                    if (packType.PackagingTypeCode == packagingTypeCode)
                    {
                        return packType.Barcodes;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Sets the item transactional packaging type info list.
        /// </summary>
        /// <param name="itemInfo">The item info.</param>
        /// <returns></returns>
        private Collection<ItemTransactionalPackagingTypeInfo> SetItemTransactionalPackagingTypeInfoList(PmxItemInfo itemInfo)
        {
            //get the packaging types qty from item
            Collection<ItemTransactionalPackagingTypeInfo> listOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

            foreach (PmxItemPackagingTypeInfo packInfo in itemInfo.ListOfPackagingTypes)
            {
                ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

                newPackInfo.PackagingTypeCode = packInfo.PackagingTypeCode;
                newPackInfo.Quantity = packInfo.Quantity;
                newPackInfo.ItemCode = itemInfo.ItemCode;

                listOfTransactionalPackagingTypeInfos.Add(newPackInfo);

            }

            return listOfTransactionalPackagingTypeInfos;
        }


        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get batch and best before date.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static GeneratedBatchAndBestBeforeDateInfo ExecuteGetBatchAndBestBeforeDate(PmxDbConnection conn, object[] parameters)
        {
            PmxGeneralProvider prov = new PmxGeneralProvider(conn);
            return prov.GetBatchAndBestBeforeDateForProduction((int)parameters[0]);
        }


        private static double? ExecuteGetMaximumQuantity(PmxDbConnection conn, object[] parameters)
        {
            PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
            return prov.GetMaximumQuantityToProduce((int)parameters[0], null, (bool)parameters[1], (string) parameters[2], (PmxItemInfo)parameters[3]);
        }

        private static string ExecuteGetStorageLocationLogisticCarriers(PmxDbConnection conn, object[] parameters)
        {
            PmxOseWarehouseProvider prov = new PmxOseWarehouseProvider(conn);
            return prov.GetStorageLocationForLogisticCarriersForLocation((string)parameters[0]);
        }

        private static InventoryItemInfo ExecuteProduceAndLock(PmxDbConnection conn, int prodOrderDocEntry, double logUnitQty, string logisticCarrierLocationCode, string logisticCarrierCode, int? existingLUID, string itemCode, string outputLocationCode, Collection<ItemTransactionalPackagingTypeInfo> currentListOfTransactionalPackagingTypeInfos, GeneratedBatchAndBestBeforeDateInfo genBatchBBDInfo, Collection<string> tempListOfSerialNumbers)
        {
            PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
			PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo> batchInfo = new PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo>();
			batchInfo.Add(itemCode, genBatchBBDInfo);

			return prov.ProduceAndLock(prodOrderDocEntry, logUnitQty, 0, logisticCarrierLocationCode, logisticCarrierCode, existingLUID, true, itemCode, outputLocationCode, currentListOfTransactionalPackagingTypeInfos, batchInfo, tempListOfSerialNumbers, null, null, null);
        }

        ///// <summary>
        ///// Executes the print events.
        ///// </summary>
        ///// <param name="conn">The conn.</param>
        ///// <param name="parameters">The parameters.</param>
        ///// <returns></returns>
        //private static int ExecutePrintEvents(PmxDbConnection conn, object[] parameters)
        //{
        //    PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

        //    return prov.PrintReportsForEvent((PmxPrintReportEventType)parameters[0], (string)parameters[1], (string)parameters[2], (int)parameters[3]);
        //}




        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }


        #endregion

        #region QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        private class BuildQuery
        {

            /// <summary>
            /// Gets the query get can be lined up locations.
            /// </summary>
            /// <param name="prodLineCode">The prod line code.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="prodOrderDocEntry">The prod order doc entry.</param>
            /// <param name="cultInfo">The cult info.</param>
            /// <returns></returns>
            internal static string GetQueryGetCanBeLinedUpLocations(string prodLineCode, string itemCode, int prodOrderDocEntry, CultureInfo cultInfo)
            {
                string secondProductionOrderTableName = "OWOR2";

                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankCode.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankName.NAME);
                strBuilder.AppendFormat(", SUM( {0}.{1} ) AS {2}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.Qty.NAME));
                strBuilder.AppendFormat(", NULL AS {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.UomDecimals.NAME));
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Uom.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.Uom.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.IngredientMustBeProduced.NAME);

                strBuilder.AppendFormat(" FROM {0}", SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.Code.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.CanBePickedForProduction.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultInfo));

                strBuilder.AppendFormat(" WHERE {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultInfo));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodLineCode, cultInfo));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, cultInfo));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, cultInfo));


                strBuilder.AppendFormat(" GROUP BY {0}.{1}, {2}.{3}, {4}.{5}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME),
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Uom.NAME));

                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME);




                strBuilder.Append(" UNION SELECT DISTINCT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.OutputLocationCode.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankCode.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankName.NAME);
                strBuilder.AppendFormat(", 0 AS {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.Qty.NAME));
                strBuilder.AppendFormat(", NULL AS {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.UomDecimals.NAME));
                strBuilder.AppendFormat(", NULL AS {0}",
                   SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.Uom.NAME));

                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.IngredientMustBeProduced.NAME);



                strBuilder.AppendFormat(" FROM {0}", SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProductionLineCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));


                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0} AS {1} ", SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME), SqlCommandHelper.EscapeTableName(secondProductionOrderTableName));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(secondProductionOrderTableName),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(secondProductionOrderTableName),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, cultInfo));




                strBuilder.AppendFormat(" WHERE {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, cultInfo));

                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.Status.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, cultInfo));


                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProductionStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Started.VALUE, cultInfo));







                strBuilder.AppendFormat(" ORDER BY {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.TankName.NAME));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the query get prod order lined up locations.
            /// </summary>
            /// <param name="prodOrderDocEntry">The prod order doc entry.</param>
            /// <param name="filterAutoSelectItems">The filter auto select items.</param>
            /// <param name="cultureInfo">The culture info.</param>
            /// <returns></returns>
            internal static string GetQueryGetProdOrderLinedUpLocations(int prodOrderDocEntry, bool? filterAutoSelectItems, CultureInfo cultureInfo)
            {
                /*
                    SELECT [OWOR].[DocNum] AS [DocNum],[WOR1].[LineNum] AS [ProdOrderLineNum],[OITM].[ItemCode] AS [ItemCode],[OITM].[ItemName] AS [ItemDescr],[OITM].[U_PMX_UOMD] AS [UomDecimals],[OITM].[InvntryUom] AS [Uom],[OITM].[U_PMX_ALUS] AS [AutoLineUpSelection],[PMX_OSSL].[Code] AS [TankCode],[PMX_OSSL].[Name] AS [TankName],[PMX_CBLU].[IngredientMustBeProduced] AS [IngredientsMustBeProduced], MAX( [WOR1].[PlannedQty] ) - MAX( [WOR1].[IssuedQty] ) - ISNULL( (
                        SELECT SUM( [PMX_INLD].[Quantity] )
                        FROM [PMX_INLD]
                        WHERE [PMX_INLD].[StorLocCode] = [PMX_OSSL].[Code] AND [PMX_INLD].[ItemCode] = [OITM].[ItemCode] AND [PMX_INLD].[BaseEntry] = 2
                    ), 0.0 ) AS [QtyStillNeeded], SUM( [PMX_INVT].[Quantity] ) AS [Qty], ISNULL( (
                        SELECT SUM( [PMX_INLD].[Quantity] )
                        FROM [PMX_INLD]
                        WHERE [PMX_INLD].[StorLocCode] = [PMX_OSSL].[Code] AND [PMX_INLD].[ItemCode] = [OITM].[ItemCode] AND [PMX_INLD].[BaseEntry] = 2
                    ), 0.0 ) AS [QtyConsumed]
                    FROM [OWOR]
                    INNER JOIN [WOR1] ON [OWOR].[DocEntry] = [WOR1].DocEntry
                    INNER JOIN [OITM] ON [WOR1].[ItemCode] = [OITM].ItemCode
                    INNER JOIN [PMX_OSPL] ON [OWOR].[U_PMX_PLCD] = [PMX_OSPL].Code
                    LEFT JOIN ( [PMX_LIUP]
                       INNER JOIN [PMX_OSSL] ON [PMX_LIUP].[LinedUpLocationCode] = [PMX_OSSL].Code AND [PMX_OSSL].[IsActive] = N'Y'
                       LEFT JOIN [PMX_INVT] ON [PMX_OSSL].[Code] = [PMX_INVT].StorLocCode
                    ) ON [PMX_OSPL].[Code] = [PMX_LIUP].ProductionLineCode AND [WOR1].[LineNum] = [PMX_LIUP].ProductionOrderLineNum AND ( [WOR1].[ItemCode] = [PMX_INVT].ItemCode OR [PMX_INVT].ItemCode IS NULL) 
                       LEFT JOIN [PMX_CBLU] ON [PMX_CBLU].[ParentCode] = [OWOR].U_PMX_PLCD AND [PMX_CBLU].[CanBeLinedUpLocationCode] = [PMX_LIUP].LinedUpLocationCode
                    WHERE [OWOR].[DocEntry] = 2 AND [WOR1].[U_PMX_HBLU] = N'Y' AND ( [OITM].[U_PMX_ALUS] = N'N' OR [OITM].[U_PMX_ALUS] IS NULL )
                    GROUP BY [OWOR].[U_PMX_PLCD], [OWOR].[DocNum], [WOR1].[LineNum], [OITM].[ItemCode], [OITM].[ItemName], [OITM].[U_PMX_UOMD], [OITM].[InvntryUom], [OITM].[U_PMX_ALUS], [PMX_OSSL].[Code], [PMX_OSSL].[Name], [PMX_CBLU].[IngredientMustBeProduced]
                    ORDER BY [WOR1].[LineNum]
                */

                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.ProdOrderLineNum.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.ItemCode.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemName.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.ItemDescr.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UomDecimals.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.UomDecimals.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.Uom.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.AutoLineUpSelection.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.AutoLineUpSelection.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankCode.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.TankName.NAME);
                strBuilder.Append(",");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME, PmxProdOrderLineUpLocationsDefinition.Columns.IngredientMustBeProduced.NAME);
                strBuilder.AppendFormat(", MAX( {0} ) - MAX( {1} ) - ISNULL( (",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.IssuedQuantity.NAME));
                {
                    strBuilder.AppendFormat("\r\n    SELECT SUM( {0} )",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.Quantity.NAME));
                    strBuilder.AppendFormat("\r\n    FROM {0}",
                        SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));
                    strBuilder.AppendFormat("\r\n    WHERE {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(((int)SboObjectType.ProductionOrder).ToString(), cultureInfo));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, cultureInfo));
                }
                strBuilder.AppendFormat("\r\n), 0.0 ) AS {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.QtyStillNeeded.NAME));
                strBuilder.AppendFormat(", SUM( {0} ) AS {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.Qty.NAME));
                strBuilder.Append(", ISNULL( (");
                {
                    strBuilder.AppendFormat("\r\n    SELECT SUM( {0} )",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.Quantity.NAME));
                    strBuilder.AppendFormat("\r\n    FROM {0}",
                        SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));
                    strBuilder.AppendFormat("\r\n    WHERE {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(((int)SboObjectType.ProductionOrder).ToString(), cultureInfo));
                    strBuilder.AppendFormat(" AND {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, cultureInfo));
                }
                strBuilder.AppendFormat("\r\n), 0.0 ) AS {0}",
                    SqlCommandHelper.EscapeColumnName(PmxProdOrderLineUpLocationsDefinition.Columns.QtyConsumed.NAME));


                strBuilder.AppendFormat("\r\nFROM {0}", SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxProductionOrderLineTable.Columns.DocEntry.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxItemTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxOseProductionLineTable.Columns.Code.NAME));


                strBuilder.Append("\r\nLEFT JOIN (");
                {
                    strBuilder.AppendFormat(" {0}", SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME));

                    strBuilder.AppendFormat("\r\n   INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                    strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                        SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME),
                        SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                        SqlCommandHelper.EscapeColumnValue(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                    strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                        SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureInfo));
                }
                strBuilder.AppendFormat("\r\n) ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxLinedUpLocationTable.Columns.ProductionLineCode.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxLinedUpLocationTable.Columns.ProductionOrderLineNum.NAME));

                strBuilder.AppendFormat("\r\nLEFT JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxProductionOrderTable.Columns.ProductionLineCode.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME));

                strBuilder.AppendFormat("\r\nLEFT JOIN {0}", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxInventoryTotalTable.Columns.StorLocCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnValue(PmxInventoryTotalTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat("\r\nWHERE {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(prodOrderDocEntry, cultureInfo));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureInfo));

                if (filterAutoSelectItems.HasValue)
                {
                    strBuilder.AppendFormat(" AND ( {0}.{1} = {2}",
                        SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                        SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.AutoLineUpSelection.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(
                        filterAutoSelectItems.Value ? SboBooleanColumnValues.Y.VALUE : SboBooleanColumnValues.N.VALUE, cultureInfo));
                    if (!filterAutoSelectItems.Value)
                    {
                        strBuilder.AppendFormat(" OR {0}.{1} IS NULL )",
                            SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                            SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.AutoLineUpSelection.NAME));
                    }
                    else
                    {
                        strBuilder.Append(" )");
                    }
                }

                strBuilder.AppendFormat("\r\nGROUP BY {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.ProductionLineCode.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderTable.Columns.DocNum.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.LineNum.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemName.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.UomDecimals.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.UOM.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.AutoLineUpSelection.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));
                strBuilder.AppendFormat(", {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME));

                strBuilder.AppendFormat("\r\nORDER BY {0}.{1}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxProductionOrderLineTable.Columns.LineNum.NAME));

                return strBuilder.ToString();
            }



            /// <summary>
            /// Gets the products from stock linked to production order.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="locationCode">The location code.</param>
            /// <param name="restLocationCode">The rest location code.</param>
            /// <returns></returns>
            public static string GetProductsFromStockLinkedToProductionOrder(int productionOrderDocEntry, string locationCode, string restLocationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(ProductDataSet.Columns.PRODUCT_CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));

                //inner join Product - inventory
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.Columns.ItemCode.NAME));

                //inner join Product - production order
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.Columns.ItemCode.NAME));


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));



                string operand = " AND ";
                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    if (string.IsNullOrEmpty(restLocationCode))
                    {
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                    }
                    else
                    {
                        //input and rest location are allowed
                        strBuilder.Append(" IN ( ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                        strBuilder.Append(", ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(restLocationCode, CultureInfo.InvariantCulture));
                        strBuilder.Append(" ) ");
                    } operand = " AND ";
                }


                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();
            }

			/// <summary>
			/// Command for getting all logistic carriers order by name
			/// </summary>
			/// <param name="onlyInStock">if set to <c>true</c> the system will get the logistic carriers that are in stock and have no products on it (LUID IS NULL)</param>
			/// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns>
			/// Command for getting all logistic carriers order by name
			/// </returns>
            public static string GetLogisticCarriers(bool onlyInStock, bool filterOutFrozenItems, DbTool dbTool)
            {

				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(ProductDataSet.Columns.PRODUCT_CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CodeBars.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));



                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));

                if (onlyInStock)
                {
                    strBuilder.Append(" INNER JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                    strBuilder.Append(" IS NULL ");
                }




                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (filterOutFrozenItems)
                {
                    Produmex.Sbo.Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(strBuilder, " AND ");
                }


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();

            }

            /// <summary>
            /// Gets the batch numbers from stock.
            /// </summary>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="location">The location.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="batchNumber">The batch number.</param>
            /// <param name="bestBeforeDate">The best before date.</param>
            /// <param name="qualityStatusCode">The quality status code.</param>
            /// <param name="checkSpecificBatchOrBBD">if set to <c>true</c> [check specific batch or BBD].</param>
            /// <param name="checkSpecificSSCC">if set to <c>true</c> [check specific SSCC].</param>
            /// <returns></returns>
            public static string GetBatchNumbersFromStock(string sscc, string location, string itemCode, string batchNumber, DateTime? bestBeforeDate, string qualityStatusCode, bool checkSpecificBatchOrBBD, bool checkSpecificSSCC)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, BatchNumberDataSet.Columns.BATCH_NUMBER.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, BatchNumberDataSet.Columns.BATCH_NUMBER2.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, BatchNumberDataSet.Columns.BEST_BEFORE_DATE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, BatchNumberDataSet.Columns.QUALITY_STATUS_CODE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, BatchNumberDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));
                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(BatchNumberDataSet.Columns.QUANTITY.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);



                string operand = " WHERE ";

                if (!string.IsNullOrEmpty(sscc))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificSSCC)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(qualityStatusCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(qualityStatusCode, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }


                if (!string.IsNullOrEmpty(location))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(location, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(itemCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(batchNumber))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificBatchOrBBD)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";

                }

                if (bestBeforeDate.HasValue)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(bestBeforeDate.Value, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificBatchOrBBD)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";

                }

                strBuilder.Append(" GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);




                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the name and the code of the  location, ordered by name.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <returns>The sql statement to get the location</returns>
            public static string GetStorageLocation(string locationCode, string warehouseCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);



                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }

                if (warehouseCode != null &&
                    warehouseCode.Trim() != "")
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


                return strBuilder.ToString();
            }


            public static string GetMovableLocations( string moveableLocationCode, string warehouseCode )
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseMovableLocationTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseMovableLocationTable.NAME, PmxOseMovableLocationTable.Columns.Code.NAME);


                strBuilder.Append(" WHERE ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                if ( warehouseCode != null &&
                    warehouseCode.Trim() != "" )
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

                }

                if ( moveableLocationCode != null &&
                    moveableLocationCode.Trim() != "" )
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(moveableLocationCode, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME);


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the production orders.
            /// </summary>
            /// <param name="areLogCarriersStoredOnOneLocation">if set to <c>true</c> [are log carriers stored on one location].</param>
            /// <param name="allowToStartProductionOrders">if set to <c>true</c> [allow to start production orders].</param>
            /// <param name="prdDocEntry">The PRD doc entry.</param>
            /// <param name="prdDocNum">The PRD doc num.</param>
            /// <returns></returns>
            public static string GetProductionOrders( bool areLogCarriersStoredOnOneLocation , bool allowToStartProductionOrders, int? prdDocEntry, int? prdDocNum)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT DISTINCT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name);
               strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name);
               strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name);

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name);
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME));
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name);
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name));
                strBuilder.Append(", (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber2.NAME, PmxProductionOrderTable.Columns.BatchNumber2.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BestBeforeDate.NAME, PmxProductionOrderTable.Columns.BestBeforeDate.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.OutputLocationCode.NAME, PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.RestLocationCode.NAME, PmxOseProductionLineTable.Columns.RestLocationCode.NAME));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name);
                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME, ProductionOrderDataSet.Columns.PMX_BATCH_NUM.Name );


                strBuilder.AppendFormat(" FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.AppendFormat(" INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);

                string itemTableForLineName = "ItemTableLine";

                strBuilder.AppendFormat(" INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(itemTableForLineName);
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, itemTableForLineName, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

                strBuilder.AppendFormat(" INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.AppendFormat(" ON {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

                strBuilder.AppendFormat(" LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME);

                strBuilder.AppendFormat(" LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" IS NOT NULL ");
                
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME);
                strBuilder.AppendFormat(" = {0}",
                SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, CultureInfo.InvariantCulture));

                if (prdDocEntry.HasValue)
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                    strBuilder.AppendFormat(" = {0}",
                    SqlCommandHelper.FormatAndEscapeColumnValue(prdDocEntry.Value, CultureInfo.InvariantCulture));
                }

                if (prdDocNum.HasValue)
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                    strBuilder.AppendFormat(" = {0}",
                    SqlCommandHelper.FormatAndEscapeColumnValue(prdDocNum.Value, CultureInfo.InvariantCulture));
                }


                if (!allowToStartProductionOrders)
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME);
                    strBuilder.AppendFormat(" = {0}",
                        SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Started.VALUE, CultureInfo.InvariantCulture));
                }
                else
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME);
                    strBuilder.AppendFormat(" <> {0}",
                        SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Closed.VALUE, CultureInfo.InvariantCulture));

                }

                //strBuilder.Append(" AND ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME);
                //strBuilder.AppendFormat(" <> {0}",
                //    SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderTypeValues.Disassembly.VALUE, CultureInfo.InvariantCulture));


                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the production orders for picking.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="areLogCarriersStoredOnOneLocation">if set to <c>true</c> [are log carriers stored on one location].</param>
            /// <returns></returns>
            public static string GetProductionOrderLinesForPicking( int productionOrderDocEntry, bool areLogCarriersStoredOnOneLocation )
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, ProductionOrderLineDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderLineDataSet.Columns.ITEMDESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderLineDataSet.Columns.UOM.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UomDecimals.NAME, ProductionOrderLineDataSet.Columns.UOM_DECIMALS.Name);

                strBuilder.Append(", ISNULL( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(", 0 ) - ISNULL( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityPicked.NAME);
                strBuilder.AppendFormat(", 0 ) AS {0}", SqlCommandHelper.EscapeColumnName(ProductionOrderLineDataSet.Columns.QUANTITY.Name));

                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.AppendFormat(" ) AS {0}", SqlCommandHelper.EscapeColumnName(ProductionOrderLineDataSet.Columns.QUANTITY_AT_INPUT.Name));


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ProductionOrderLineDataSet.Columns.IS_OPTIONAL.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);

                strBuilder.AppendFormat(" LEFT JOIN {0}", SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);

                strBuilder.AppendFormat(" LEFT JOIN {0}", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));


                strBuilder.Append(" AND ( ISNULL(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(", 0) - ISNULL( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityPicked.NAME);
                strBuilder.Append(", 0))  > 0 ");

                strBuilder.Append(" AND (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" OR ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
                strBuilder.Append(" IS NULL ) ");

                if ( areLogCarriersStoredOnOneLocation )
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" GROUP BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UomDecimals.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityPicked.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);


                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

                return strBuilder.ToString();
            }

            internal static string GetInfoForInStockOrAdvanceNoticeSSCC( string sscc )
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat( "SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ,PmxInventoryTotalTable.NAME + "_" + PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME) );
                strBuilder.AppendFormat( ", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias( PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME, PmxAdvanceShippingNoticeTable.NAME + "_" + PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME ) );
                strBuilder.AppendFormat( ", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias( PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.Quantity.NAME, PmxAdvanceShippingNoticeTable.NAME + "_" + PmxAdvanceShippingNoticeTable.Columns.Quantity.NAME ) );

                strBuilder.AppendFormat( "\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName( PmxLogisticUnitIDTable.NAME ) );

                strBuilder.AppendFormat( "\r\n LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName( PmxInventoryTotalTable.NAME ) );
                strBuilder.AppendFormat( " ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME ) );

                strBuilder.AppendFormat( "\r\n LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName( PmxAdvanceShippingNoticeTable.NAME ) );
                strBuilder.AppendFormat( " ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName( PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME ) );

                strBuilder.AppendFormat( "\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName( PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME ),
                    SqlCommandHelper.FormatAndEscapeColumnValue( sscc, CultureInfo.InvariantCulture ) );
                strBuilder.AppendFormat( " AND ( {0} IS NOT NULL OR {1} IS NOT NULL )",
                    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ),
                    SqlCommandHelper.EscapeTableColumnName( PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME ) );

                return strBuilder.ToString();
            }
        }

        #endregion
    }
}
